/*jslint node: true */
'use strict';

var meta = require('./package.json'),
    gutil = require('gulp-util'),
    through = require('through2'),
    fs = require('fs'),
    error = false,
    errorBreak = false;


/** 
 * Validation Error Printout
 * abort = true => breaks build if flag is set
 */
function validationError(tag, mistake, guideline, abort) {

    error = true;

    if (abort) {
        gutil.log('[' + guideline + '] ' + gutil.colors.white(tag + ": ") + gutil.colors.red(mistake));
        errorBreak = true;
    } else {
        gutil.log('[' + guideline + '] ' + gutil.colors.white(tag + ": ") + gutil.colors.yellow(mistake));
    }
}

/**
 * Rules:
 * One line per information: Summary, Author, Date, Copyright, 
 * No empty values
 */
function infoOk(info, lines) {

    var ok = false;
    var next = false;

    lines.some(function (line) {
        if (line.includes(info)) {

            next = false;

            if ((line.length - info.length) >= 2) {
                ok = true;
            } else {
                next = true;
            }
        } else if (next && line.length > 0) {
            ok = true;
        }
    });
    return ok;
}

function checkHeader(comment) {

    if (comment.substr(comment.search(/\S|$/)) !== '') {

        var lines = comment.split(/\r?\n/),
            infos = ["Summary", "Author", "Date", "Copyright"];

        infos.some(function (info) {
            if (!infoOk(info, lines)) {
                validationError('Header', 'Required field "' + info + '" does not exist or is empty', 'CI-FE-1001', true);
            } else if (info === 'Copyright') {
                if (comment.indexOf('Robert Bosch GmbH') <= 0) {
                    validationError('Header', 'Copyright value "Robert Bosch GmbH" is missing or incorrect', 'CI-FE-1001', true);
                }
            }
        });
    } else {
        validationError('Header', 'No Copyright Header found', 'CI-FE-1001', true);
    }
}

module.exports = function validate(options) {

    if (!options) {
        options = {};
    }

    var errorOutputList = [];

    function transformFunction(file, encoding, callback) {

        var filename = file.path.replace(/^.*[\\\/]/, '');

        if (filename.charAt(0) !== '_') {

            fs.readFile(file.path, 'utf8', function (err, str) {

                error = false;

                if (!str) {
                    validationError('', 'File is empty', '[CI-FE-1001]', false);
                } else {
                    var comment = (str.substr(str.indexOf("/*") + 2, (str.indexOf("*/") - str.indexOf("/*") - 2)));

                    checkHeader(comment);

                    if (error) {

                        if (comment.substr(comment.search(/\S|$/)) !== '') {
                            gutil.log(gutil.colors.red(comment));
                        } else {
                            gutil.log('[CI-FE-1001] Header: ' + gutil.colors.yellow('Declare partials by starting the filename with "_"'));
                        }
                        gutil.log(gutil.colors.cyan(file.path));
                        if (options.abortOnError && errorBreak) {
                            gutil.log(gutil.colors.red("Critical validation error -> ABORT"));
                            process.exit(1);
                        }
                        gutil.log();
                    }

                }
            });
        }

        return callback(null, file);
    }

    function errorOutput(callback) {
        if (errorOutputList.length > 0) {
            this.emit('error', new gutil.PluginError(meta.name, '\n' + errorOutputList.join('\n\n') + '\n', {
                showStack: false
            }));
        }
        callback();
    }

    return through.obj(transformFunction, errorOutput);
};
